/* Check to see if any user has failed to log on more than a certain
 * number of times.
*/

/* This may look unnecessarily complicated but I am trying to avoid using
 * getpwpwent() and the linked list searcher more than is needed.
 */
 
#include "bigb.h"
#include <sys/stat.h>

PRIVATE void check_all_u(void);
PRIVATE void check_specific_u(void);
PRIVATE void check_all_except_u(void);
PRIVATE void check_some_except_u(void);
PRIVATE void check_this_user(struct pr_passwd *);

#ifdef QUICK
PRIVATE int check_user_changes(void);
PRIVATE char changed[26];
#endif

PUBLIC void check_users(void)
{
#ifdef QUICK
    if (check_user_changes() == FALSE)
        return;     /* Nothing has happened */
#endif

    switch(user_check) {
        case CHECK_ALL      : check_all_u();
                            break;
        case CHECK_SPECIFIC : check_specific_u();
                            break;
        case CHECK_EXCLUDE  : check_all_except_u();
                            break;
        case CHECK_NONE     : /* Nothing required */
                            break;
        default             : check_some_except_u();
                            break;
    }
}

/* Check every user the system knows of */
PRIVATE void check_all_u(void)
{
    struct pr_passwd    *user;

    while ((user=getprpwent()) != (struct pr_passwd *) NULL)
        check_this_user(user);
    endprpwent();
}

/* Check a specific list of users */
PRIVATE void check_specific_u(void)
{
    struct member       *ptr1;
    struct pr_passwd    *user;

    ptr1 = users_to_check;
    if (ptr1 == MNULL)
        return; /* No users need to be checked */

    do {
#ifdef QUICK
        if (changed[*(ptr1->data)-'a'] == 'y')
#endif
            if ((user=getprpwnam(ptr1->data)) != (struct pr_passwd *) NULL)
                check_this_user(user);
    } while ((ptr1=ptr1->next) != MNULL);
    endprpwent();
}

/* Check all the users except those specifically excluded */
PRIVATE void check_all_except_u(void)
{
    struct pr_passwd    *user;
    
    while ((user=getprpwent()) != (struct pr_passwd *) NULL) {
#ifdef QUICK
        if (changed[*(user->ufld.fd_name)-'a'] == 'y')
#endif
            if (!find_in_list(users_to_exclude,user->ufld.fd_name))
                check_this_user(user);
    };
    endprpwent();
}

/* The most general case: check all specified users except those
 * in the exclusion file.
 */
PRIVATE void check_some_except_u(void)
{
    struct member   *ptr1;
    struct pr_passwd    *user;

    ptr1 = users_to_check;
    if (ptr1 == MNULL)
        return; /* No users need to be checked */

    do {
#ifdef QUICK
        if (changed[*(ptr1->data)-'a'] == 'y')
#endif
            if ((user=getprpwnam(ptr1->data)) != (struct pr_passwd *) NULL)
                if (!find_in_list(users_to_exclude,user->ufld.fd_name))
                    check_this_user(user);
    } while ((ptr1=ptr1->next) != MNULL);
    endprpwent();
}

/* Check the user data given */
PRIVATE void check_this_user(struct pr_passwd *user)
{
    struct pr_field *data;

    data  = &(user->ufld);

    if (data->fd_slogin >= data->fd_ulogin)
        return; /* Last login was successful */
        
    if (last_checked > data->fd_ulogin)
        return;     /* Not failed used since last check */
    
    if (data->fd_nlogins >= max_login_failures) {
        sprintf(temp, "%s: %d consecutive failures by %s on %s",
            date(data->fd_ulogin),
            data->fd_nlogins,
            data->fd_name,
            find_relevant_terminal(FALSE, data->fd_uid, data->fd_ulogin));
        print_warning(temp);
    }
}

#ifdef QUICK

/* The data on individual users is held in 26 seperate directories below
 * USERFILES, these directories are called a,b,c,d,....,z and the data on
 * user mtp for example would be in subdirectory m.
 * When any of the data in these directories is changed the modification
 * time of the directory gets set to the current time.  So to quickly check
 * if any user who's user name begins with 'm' has logged in all that needs
 * doing is to check the modification time of USERFILES/m
*/

/* This sets up an array of 26 characters ( one per letter ) to be 'y' or
 * 'n' depending if any user who's username begins with the appropriate letter
 * in the alphabet has had his/her/it's protected password data changed.
 * The subroutine also returns TRUE if at least one entry has been changed
 * or FALSE if nothing has changed.
*/
PRIVATE int check_user_changes(void)
{
    struct stat buf;
    int         some_change = FALSE;
    int         i;

    for (i=0 ; i<26 ; i++) {
        sprintf(temp,"%s%c",USERFILES,(char) ('a'+i));
        if (stat(temp,&buf) == 0) {
            if (last_checked >= buf.st_mtime) {
                /* No change since it was last checked */
                changed[i] = 'n';
            } else {
                /* Something in here has been changed */
                changed[i] = 'y';
                some_change = TRUE;
            }
        } else {
            /* Cannot access the relevant file - this means that the data on
             * these users cannot be read so there is no point in checking
             * for login failures, the getprpwent() call would fail */
            max_login_failures = -1;    /* Stop user checking */
            changed[i] = 'n';
        }
    }
    return(some_change);
}
#endif
