/* This reads in and processes the configuration file, it looks for lines
 * of the form:
 * parameter=value
 * where the possible parameters and their types are defined in the
 * parameters[] array.
*/

#include "bigb.h"
#include "defaults.h"
#include <string.h>

PRIVATE void select_defaults(void);
PRIVATE int set_options(void);
PRIVATE void do_config(void);
PRIVATE void set_channel(const char *, FILE **);

/* Make processing the parameter file a little more elegant by using a lookup
 * table of parameters to look for, their address, type and default value.
*/
struct param {
    const char  name[16];       /* keyword to look for */
    void        *address;       /* Address to put the data */
    flag_t      param_type;     /* What type the data is */
    const char  **def;          /* The default data as a string */
    };

#define TYPE_INT    0   /* This parameter is an integer */
#define TYPE_STRING 1   /* This parameter is a string */
#define NUM_PARAMS  11  /* There are NUM_PARAMS parameters to set */

/* Define the names, addresses, types and default values for all parameters */
PRIVATE struct param parameters[] = {
    { "terminalfaults", &max_term_failures, TYPE_INT,    &def_termfaults },
    { "userfaults", &max_login_failures,    TYPE_INT,    &def_userfaults },
    { "checkroot",  checkroot,              TYPE_STRING, &def_checkroot },
    { "sleeptime",  &sleeptime,             TYPE_INT,    &def_sleeptime },
    { "nicevalue",  &nicevalue,             TYPE_INT,    &def_nicevalue },
    { "errorfile",  error_file_name,        TYPE_STRING, &def_error_file_name },
    { "errorout",   error_display_name,     TYPE_STRING, &def_error_display_name }, 
    { "badterms",   terminal_check_file,    TYPE_STRING, &def_terminals_to_check }, 
    { "goodterms",  terminal_nocheck_file,  TYPE_STRING, &def_terminals_not_to_check }, 
    { "badusers",   user_check_file,        TYPE_STRING, &def_users_to_check }, 
    { "goodusers",  user_nocheck_file,      TYPE_STRING, &def_users_not_to_check }
    };

/* Read the configuration file in and process it */
PUBLIC int read_config(void)
{
    /* Set all parameters to their default values */
    select_defaults();

    /* Set all parameters using the configuration file */
    set_options();

    /* Do any processing needed on the parameters */
    do_config();
}

/* Set all parameters to their default values */
PRIVATE void select_defaults(void)
{
    int     i;

    for (i=0 ; i<NUM_PARAMS ; i++ ) {
        switch (parameters[i].param_type) {
            case TYPE_INT:
                *((int *) parameters[i].address) = atoi(*(parameters[i].def));
                break;
            case TYPE_STRING:
                strcpy(parameters[i].address,*(parameters[i].def));
                break;
        }
    }   
}

PRIVATE int set_options(void)
{
    FILE    *fp;
    int     equals;
    int     i;

    /* Try to open the configuration file */
    if ((fp=fopen(config_file,"r")) == FNULL) {
        return(0);  /* No configuration file so use defaults */
    }

    /* Go through the entire  configuration file */
    while (!feof(fp)) {
        /* Read the next line into temp */
        if (fgets(temp, PATH_MAX, fp) != CNULL) {
            /* Tidy it up a bit */
            temp[PATH_MAX] = '\0';
            neaten(temp);
            /* Look for an '=' in the line ( means it is a valid line ) */
            if ((equals=strcspn(temp,"=")) != strlen(temp)) {
                /* Found an '=' so compare the line to all known parameters */
                for (i=0 ; i<NUM_PARAMS ; i++) {
                    /* Does this parameter name match? */
                    if (strncmp(temp,parameters[i].name,equals) == 0) {
                        /* It matches so set the relevant parameter */
                        switch(parameters[i].param_type) {
                            case TYPE_INT: /* It is an integer field */
                                *((int *) parameters[i].address)=atoi(temp+equals+1);
                                break;
                            case TYPE_STRING: /* It is a text field */
                                strcpy(parameters[i].address,temp+equals+1);
                                break;
                        } /*  switch */
                        break;
                    } /* strncmp */
                } /* for */
            } /* strcspn */
        } /* fgets */
    }; /* while */
    fclose(fp);
    return(0);
}

PRIVATE void do_config(void)
{
    /* Do the requested niceing */
    nice(nicevalue);

    /* Set a boolean TRUE/FALSE from 'yes'/'no' */
    if (strcmp("yes",checkroot) == 0)
        check_root_selected = TRUE;
    else
        check_root_selected = FALSE;

    /* Set up the output channels */
    set_channel(error_display_name, &error_display_chan);
    set_channel(error_file_name, &error_file_chan);

    /* Check for authorization to read user data */
    /* WARNING! This will fail if the program is not sgid auth */
    if ((getgid() != getegid()) && (!authorized_user("auth"))) {
        max_login_failures = -1;            /* Cannot read user data */
        fputs("bigb: Insufficient authorization to check user accounts\n"
            ,error_display_chan);
    }
}

/* Change the channel specified to be associated with the new file */
PRIVATE void set_channel(const char *file_name, FILE **chan)
{
    if (*chan == FNULL || *chan == stdout) {
        fflush(stdout);     /* Paranoia? */
        if (*file_name == '\0')
            *chan = stdout;
        else
            *chan=fopen(file_name,"a+");
    } else {
        if (*file_name == '\0')
            *chan = stdout;
        else
            *chan=freopen(file_name,"a+",*chan);
    }

    if (*chan == FNULL) {
        /* Probably no such file - default to stdout */
        sprintf(temp,"Cannot open '%s'",file_name);
        perror(temp);
        fputs("Defaulting to stdout\n",stderr);
        *chan = stdout;
    }
}
